<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Social Login Hyva Compatibility
 */
declare(strict_types=1);

namespace Amasty\SocialLoginHyva\ViewModel\Popup;

use Amasty\SocialLogin\Model\ConfigData;
use Amasty\SocialLogin\Model\Source\RedirectType;
use Magento\Framework\Url;
use Magento\Framework\View\Element\Block\ArgumentInterface;
use Magento\Framework\View\LayoutInterface;

class Redirect implements ArgumentInterface
{
    /**
     * Amasty configuration model
     *
     * @var ConfigData
     */
    private ConfigData $configData;

    /**
     * Layout instance
     *
     * @var LayoutInterface
     */
    private LayoutInterface $layout;

    /**
     * Utility for building store URL
     *
     * @var Url
     */
    private Url $urlBuilder;

    /**
     * Ajax popup must not perform ajax reload on pages with layout handles provided in this array
     *
     * @var array
     */
    private array $handlesToForceReload = [];

    /**
     * @param ConfigData $configData
     * @param LayoutInterface $layout
     * @param Url $urlBuilder
     * @param array $handlesToForceReload
     */
    public function __construct(
        ConfigData $configData,
        LayoutInterface $layout,
        Url $urlBuilder,
        array $handlesToForceReload = []
    ) {
        $this->configData = $configData;
        $this->layout = $layout;
        $this->urlBuilder = $urlBuilder;
        $this->handlesToForceReload = $handlesToForceReload;
    }

    /**
     * Retrive the redirection type after successful form submitting
     *
     * Based on configuration entry and current page
     *
     * @return int
     */
    public function getRedirectType(): int
    {
        $redirectTypeFromConfig = $this->configData->getRedirectType();
        if ($redirectTypeFromConfig === RedirectType::AJAX_REFRESH && $this->isForceReload()) {
            return RedirectType::REFRESH_PAGE;
        }
        return $redirectTypeFromConfig;
    }

    /**
     * Retrieve the configuration entry value for custom redirect URL
     *
     * @return string
     */
    public function getRedirectUrl(): string
    {
        $configValue = $this->configData->getConfigValue('general/custom_url');

        return $this->urlBuilder->getUrl(ltrim($configValue, '\/'));
    }

    /**
     * Check if Ajax reload can be performed on current page
     *
     * @return bool
     */
    public function isForceReload(): bool
    {
        $layoutHandles = $this->layout->getUpdate()->getHandles();

        foreach ($layoutHandles as $handle) {
            if (in_array($handle, array_values($this->handlesToForceReload))) {
                return true;
            }
        }

        return false;
    }
}
